/*
    ---------------------------------------------------------------------- copyright and license ---

    File: src/layouter.hpp

    Copyright 🄯 2014, 2016—2017, 2019—2022 Van de Bugger.

    This file is part of Tapper.

    Tapper is free software: you can redistribute it and/or modify it under the terms of the GNU
    General Public License as published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    Tapper is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without
    even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    General Public License for more details.

    You should have received a copy of the GNU General Public License along with Tapper.  If not,
    see <https://www.gnu.org/licenses/>.

    SPDX-License-Identifier: GPL-3.0-or-later

    ---------------------------------------------------------------------- copyright and license ---
*/

/** @file
    `layouter_t` abstract class interface.

    @sa layouter.cpp
**/

#ifndef _TAPPER_LAYOUTER_HPP_
#define _TAPPER_LAYOUTER_HPP_

#include "base.hpp"

#include "settings.hpp"
#include "types.hpp"

namespace tapper {

// -------------------------------------------------------------------------------------------------
// layouter_t
// -------------------------------------------------------------------------------------------------

class layouter_t;
using layouter_p = ptr_t< layouter_t >;

/**
    Layouter abstract interface and factory.

    "Layouter" is a shorter name of "Layout manager".
**/
class layouter_t {

    public:

        /** Layouter exceptions. **/
        class error_t: public std::runtime_error {
            public:
                using std::runtime_error::runtime_error;
        }; // class error_t

    public:

        /** Layouter factory. Returns pointer to a new layouter of the specified type. **/
        static layouter_t * create( settings_t::layouter_t type );

        virtual ~layouter_t() = default;

        /**
            Any descendant should override this method. The method should return layouter type
            name, e. g. "GNOME", "KDE" or "Xkb".
        **/
        virtual string_t type() = 0;

        bool bell();

        /**
            Sets bell status: if `true`, the bell is enabled, and disabled otherwise.
        **/
        void set_bell( bool bell );

        /**
            Returns keyboard delay in milliseconds.
        **/
        time_t repeat_delay();

        /**
            Returns the range of valid layout indices for the layouter. Range represents
            "theoretical" lower and upper bounds, not all indices in the range may represent
            actually configured and ready-to-use layouts.
        **/
        virtual layout_range_t layout_range();

        /**
            Returns the set of layout indices for the layouter. In contrast to `layout_range`, all
            the indices in the set represents actully available layouts at some moment (e. g. at
            the moment of layoter initialization).
        **/
        layouts_t layouts();

        /**
            Returns the name of layout for given layout index.
        **/
        string_t layout_name( layout_t layout );

        /**
            Returns the full name of layout for given layout index. Full layout name consists of
            layout index, colon, and layout name (colon is not added if layout name is empty).
        **/
        string_t layout_full_name( layout_t layout );

        /**
            Returns the index of layout for given layout name.
        **/
        layout_t layout( string_t const & name );

        /**
            The method activates the given layout and ring the bell (if `_bell_` is `true` and
            layouter is capable to do it).

            Descentands are expected to override the method.
        **/
        virtual void activate(
            layout_t layout     ///< Index of the layout to activate.
        ) = 0;

    protected:

        /**
            The returns keyboard repeat delay (amount of time the user has to hold a key down
            before the system will generate repeating key press events). Note that `repeat_delay`
            is just a front-end, the actual functionality is provided by this method.

            Non-dummy descendatns are expected to override the method.
         **/
        virtual time_t  _get_repeat_delay();

        /**
            The method returns list of layout names know to the layouter. To avoid repeated
            requests the result of the method is cached, see `_layout_names` and `_layout_names_`.

            Non-dummy descendatns are expected to override the method.
        **/
        virtual strings_t _get_layout_names();

        /**
            The method returns `true` if the layouter can ring the bell, and `false` otherwise.
            Default implementation returns `false`. If the decendant can ring, it should override
            the method.

            Non-dummy descendatns are expected to override the method.
        **/
        virtual bool _can_ring();

    private:

        strings_t const & _layout_names();

    private:

        bool        _bell_ { false };
            ///< Bell status: bell is enabled (if `true`) or disabled (if `false`).
        time_t      _repeat_delay_ { 0 };
            ///< Cached repeat delay returned by `_get_repeat_delay`.
        strings_p   _layout_names_;
            ///< Cached layout names returned by `_get_layout_names`.

}; // class layouter_t

/// Namespace for concrete layouters.
namespace layouter {
}; // namespace layouter

}; // namespace tapper

#endif // _TAPPER_LAYOUTER_HPP_

// end of file //
