/*
    ---------------------------------------------------------------------- copyright and license ---

    File: src/tapper.hpp

    Copyright 🄯 2014, 2016—2017, 2019—2022 Van de Bugger.

    This file is part of Tapper.

    Tapper is free software: you can redistribute it and/or modify it under the terms of the GNU
    General Public License as published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    Tapper is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without
    even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    General Public License for more details.

    You should have received a copy of the GNU General Public License along with Tapper.  If not,
    see <https://www.gnu.org/licenses/>.

    SPDX-License-Identifier: GPL-3.0-or-later

    ---------------------------------------------------------------------- copyright and license ---
*/

/** @file
    `tapper_t` class interface.

    @sa tapper.cpp
**/

#ifndef _TAPPER_TAPPER_HPP_
#define _TAPPER_TAPPER_HPP_

#include "base.hpp"

#include "emitter.hpp"
#include "layouter.hpp"
#include "listener.hpp"
#include "settings.hpp"
#include "types.hpp"

namespace tapper {

// -------------------------------------------------------------------------------------------------
// tapper_t
// -------------------------------------------------------------------------------------------------

/**
    A component that orchestrates listener, layouter and emitter to do the work.

    Tapper listens for keyboard events (with help from listener) detects taps, and activates
    keyboard layouts (with help from layouter).
**/
class tapper_t: public object_t {

    public:

        explicit tapper_t(
            listener_t &          listener,
            layouter_t &          layouter,
            emitter_t &           emitter,
            assignments_t const & assignments,
            bool                  show_taps = false
        );
        ~tapper_t();
        void start();
        void stop();

    private:

        using event_t = listener_t::event_t;

        void _on_event( event_t const & event );
        void _on_tap( key_t key );

    private:

        listener_t &  _listener;
        layouter_t &  _layouter;
        emitter_t &   _emitter;
        assignments_t _assignments;
        bool          _show_taps { false };
        time_t        _repeat_delay { 0 };
            // TODO: Update it?

        key_t::range_t _key_range;

        /**
            Keyboard state. It maps keys to boolean values. `true` means the corresponding key is
            pressed now, `false` means the key is not pressed.
        **/
        std::vector< bool > _key_state;

        /**
            Number of currently pressed keys. The number of currently pressed keys can be achieved
            from `_key_state`, but scanning the `_key_state` is rather slow. Maintaining number of
            pressed keys is simple and fast.
        **/
        uint_t _pressed_keys { 0 };

        /**
            Key code of the last pressed key or 0. Used to detect taps: if code of released key
            equals to code of the last pressed key, it could be a tap.
        **/
        key_t _last_key;

        /**
            Time of the last key press event, which code was saved in `last_pressed_key`.
            Meaningful only if `last_pressed_key` is not 0.
        **/
        time_t _pressed_at { 0 };

}; // class tapper_t

}; // namespace tapper

#endif // _TAPPER_TAPPER_HPP_

// end of file //
